/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.communication.jndi.client;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

/**
 * The JNDI service provides facilities to communicate with a JNDI server.
 * <p>
 * It provides a decorated {@code InitialContext}, {@link InitialContextDecorator},
 * in order to manipulate a unique {@code InitialContext}.
 * </p>
 * 
 * @author Adrien Louis - EBM WebSourcing
 * @author Christophe HAMERLING - EBM WebSourcing
 */
public interface JNDIService {
    
    // Property to provide logger to the custom initial context factory that need it.
    public final static String JAVA_NAMING_FACTORY_LOGGER = "java.naming.factory.logger";

    /**
     * Name of the JNDI user context
     */
    public static final String USERS_REF = "users";

    /**
     * Get the initial JNDI context.
     * 
     * @return {@link InitialContextDecorator}
     * @throws NamingException
     */
    InitialContext getInitialContext() throws NamingException;

    /**
     * Return a JNDI specific Context for Components
     * 
     * @return
     * @throws NamingException
     */
    public Context getUsersContext() throws NamingException;

}
