/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.communication.jndi.client.naming;

import javax.naming.Binding;
import javax.naming.NameClassPair;
import javax.naming.NamingException;

import org.ow2.petals.microkernel.communication.jndi.agent.JNDIAgentService;
import org.ow2.petals.microkernel.communication.jndi.agent.msg.request.RegistryRequest;
import org.ow2.petals.microkernel.communication.jndi.agent.msg.response.AckResponse;
import org.ow2.petals.microkernel.communication.jndi.agent.msg.response.CreateSubcontextResponse;
import org.ow2.petals.microkernel.communication.jndi.agent.msg.response.ListBindingsResponse;
import org.ow2.petals.microkernel.communication.jndi.agent.msg.response.ListResponse;
import org.ow2.petals.microkernel.communication.jndi.agent.msg.response.LookupLinkResponse;
import org.ow2.petals.microkernel.communication.jndi.agent.msg.response.LookupResponse;
import org.ow2.petals.microkernel.communication.jndi.agent.msg.response.RegistryResponse;

/**
 * @author Roland NAUDIN - EBM WebSourcing
 */
public class JNDILocalConnection implements NamingConnection {

    /**
     * The local reference of the JNDI agent
     */
    private JNDIAgentService jndiAgentService;

    public JNDILocalConnection(JNDIAgentService jndiAgentService) {
        this.jndiAgentService = jndiAgentService;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.petals.communication.jndi.agent.naming.NamingConnection#send(org.ow2.petals.communication.jndi.agent.msg.request.RegistryRequest)
     */
    public Object send(RegistryRequest request) throws NamingException {
        RegistryResponse response = null;

        // process the request
        Object object = jndiAgentService.processRequest(request);

        // send the response
        switch (request.getType()) {
            /*
             * BIND
             */
            case bind:
                response = new AckResponse();
                break;
            /*
             * CREATE SUBCONTEXT
             */
            case createSubcontext:
                response = new CreateSubcontextResponse((String) object);
                break;
            /*
             * DESTROY SUBCONTEXT
             */
            case destroySubcontext:
                response = new AckResponse();
                break;
            /*
             * LIST
             */
            case list:
                response = new ListResponse((NameClassPair[]) object);
                break;
            /*
             * LIST BINDINGS
             */
            case listBindings:
                response = new ListBindingsResponse((Binding[]) object);
                break;
            /*
             * LOOKUP
             */
            case lookup:
                response = new LookupResponse(object);
                break;
            /*
             * LOOKUP LINK
             */
            case lookupLink:
                response = new LookupLinkResponse(object);
                break;
            /*
             * PING
             */
            case ping:
                response = new AckResponse();
                break;
            /*
             * REBIND
             */
            case rebind:
                response = new AckResponse();
                break;
            /*
             * RENAME
             */
            case rename:
                response = new AckResponse();
                break;
            /*
             * UNBIND
             */
            case unbind:
                response = new AckResponse();
                break;
            default:
                break;
        }

        return response.getArg1();
    }

}
