/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.extension;

import java.util.ArrayList;
import java.util.List;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.microkernel.api.extension.InstallationExtension;
import org.ow2.petals.microkernel.api.util.LoggingUtil;

/**
 * The implementation of installation extension manager
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = InstallationExtensionsManager.class))
public class InstallationExtensionsManagerImpl implements InstallationExtensionsManager {

    /**
     * The installation extensions deployed.
     */
    final List<InstallationExtension> extensions = new ArrayList<InstallationExtension>();

    private LoggingUtil log;

    /**
     * {@inheritDoc}
     */
    public void register(final InstallationExtension installationExtension) {
        this.log.call();
        this.extensions.add(installationExtension);
    }

    /**
     * {@inheritDoc}
     */
    public void deregister(final InstallationExtension installationExtension) {
        this.log.call();
        this.extensions.remove(installationExtension);
    }

    /**
     * {@inheritDoc}
     */
    public List<InstallationExtension> getExtensions() {
        this.log.call();
        return this.extensions;
    }

    /**
     * Starts the installation extensions manager.
     */
    @LifeCycle(on = LifeCycleType.START)
    protected void start() {
        this.log = new LoggingUtil(
                java.util.logging.Logger
                        .getLogger(InstallationExtensionsManager.COMPONENT_LOGGER_NAME));
        this.log.call();

    }

    /**
     * Stops the installation extensions manager.
     */
    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.call();
    }

}
