/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.deployment.undeploy;

import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.jbi.management.Task;
import org.ow2.petals.microkernel.api.jbi.management.XMLResult;
import org.ow2.petals.microkernel.api.jbi.management.XMLResult.MessageType;
import org.ow2.petals.microkernel.api.jbi.management.XMLResult.TaskResult;

/**
 * This task create framework task result
 * <p>
 * Inputs : entity name {@link String}<br />
 * Outputs : {@link XMLResult}<br />
 * The task has a robust approach: it tries to do all the subtasks
 * and has the undo capability.
 * </p>
 * 
 * @author Olivier Fabre - EBM WebSourcing
 * @author Frederic Gardes - EBM WebSourcing
 */
public class CreateXMLUndeploymentResultTask implements Task {

    public void execute(Context context) throws Exception {
        if (context.getEntityName() != null) {
            XMLResult xmlResult = new XMLResult("undeploy");
            if (xmlResult != null) {
                this.createFrameworkTaskResult(context.getEntityName(), xmlResult);
                context.setXmlResult(xmlResult);
            }
        }
    }

    /**
     * Create the framework task result of the xml result message. Framework
     * task result is always a success because we decide that in other cases, an
     * exception is thrown.
     * 
     * @param saName
     *            the service assembly name
     * @param result
     *            framework task result
     */
    protected void createFrameworkTaskResult(String saName, XMLResult result) {
        result.addFrameworkTaskResult(TaskResult.SUCCESS, MessageType.INFO,
                "Successfully undeployed service assembly : {1}", new String[] { saName },
                null);
    }

    public void undo(Context context) throws Exception {
        context.setXmlResult(null);
    }
}
