/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.deployment.undeploy;

import org.ow2.petals.microkernel.api.container.ServiceAssemblyLifeCycle;
import org.ow2.petals.microkernel.api.jbi.management.AdminService;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.jbi.management.Task;

/**
 * Retrieve service assembly lifecycle.
 * <p>
 * Inputs : entity name {@link String} <br/>
 * Outputs : {@link ServiceAssemblyLifeCycle}<br />
 * The task has a robust approach: it tries to do all the subtasks and has undo
 * capability.
 * </p>
 * 
 * @author Olivier Fabre - EBM WebSourcing
 * @author Frederic Gardes - EBM WebSourcing
 */
public class RetrieveSALifeCycleTask implements Task {

    /**
     * JBI Container :: JMX Agent Admin
     */
    protected AdminService adminService;

    public RetrieveSALifeCycleTask(AdminService adminService) {
        super();
        this.adminService = adminService;
    }

    public void execute(Context context) throws Exception {
        String name = context.getEntityName();
        if (name != null) {
            final ServiceAssemblyLifeCycle saLifeCycle = this
                    .retrieveServiceAssemblyLifeCycle(name);
            if (saLifeCycle != null) {
                context.setSaLifeCycle(saLifeCycle);
            }
        }
    }

    /**
     * Retrieve the {@link ServiceAssemblyLifeCycle} matching the given service
     * assembly name.
     * 
     * @param saName
     *            the service assembly name
     * @return the retrieved {@link ServiceAssemblyLifeCycle}
     * @throws Exception
     *             if no service assembly can be retrieved
     */
    protected ServiceAssemblyLifeCycle retrieveServiceAssemblyLifeCycle(String saName) {
        final ServiceAssemblyLifeCycle saLifeCycle = this.adminService
                .getServiceAssemblyByName(saName);
        return saLifeCycle;
    }

    public void undo(Context context) throws Exception {
        context.setSaLifeCycle(null);
    }
}
