/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.installation.uninstall.component;

import java.util.List;

import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.extension.InstallationExtension;
import org.ow2.petals.microkernel.api.extension.InstallationExtensionException;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.jbi.management.task.AbstractFileManipulationTask;
import org.ow2.petals.systemstate.generated.Component;

/**
 * This task moves a component archive from the installed directory to the uninstalled directory.
 * <p>
 * Inputs : {@link ComponentState}<br />
 * The task has got a robust approach : it tries to do all the subtasks<br />
 * and doesn't have the undo capability.
 * </p>
 * 
 * @author Olivier Fabre - EBM WebSourcing
 * @author Frederic Gardes - EBM WebSourcing
 */
public class RemoveComponentArchiveTask extends AbstractFileManipulationTask {

    final List<InstallationExtension> extensions;

    public RemoveComponentArchiveTask(final LoggingUtil log,
            final ContainerConfiguration containerConfiguration,
            final List<InstallationExtension> extensions) {
        super(log, containerConfiguration);
        this.extensions = extensions;
    }

    /*
     * (non-Javadoc)
     * @see org.ow2.petals.jbi.management.Task#execute(org.ow2.petals.jbi.management.Context)
     */
    public void execute(Context context) throws Exception {
        Component componentStateHolder = context.getComponentStateHolder();
        if (componentStateHolder != null) {

            for (final InstallationExtension extension : this.extensions) {
                try {
                    extension.onUninstallation(componentStateHolder.getArchiveName());
                } catch (final InstallationExtensionException e) {
                    this.log.warning("An error occurs notifying uninstallation", e);
                }
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#undo(java.util.Map)
     */
    public void undo(Context context) throws Exception {
        // TODO : recreate the component archive
    }
}
