/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.installation.uninstall.sl;

import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.jbi.management.AdminService;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.jbi.management.ManagementException;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.container.ContainerService;
import org.ow2.petals.microkernel.jbi.management.task.AbstractLoggableTask;

/**
 * This tasks removes the shared library classloader from Petals platform.
 * <p>
 * Inputs : shared lib name<br />
 * The task has got a robust approach : it tries to do all the subtasks<br />
 * and doesn't have the undo capability.
 * </p>
 * 
 * @author Olivier Fabre - EBM WebSourcing
 * @author Frederic Gardes - EBM WebSourcing
 */
public class RemoveSLClassloaderTask extends AbstractLoggableTask {
    /**
     * JBI Admin Service :: JMX Agent Admin
     */
    private final AdminService adminService;

    /**
     * The Petals Container
     */
    protected ContainerService containerService;

    public RemoveSLClassloaderTask(final LoggingUtil log, final AdminService adminService,
            final ContainerService containerService,
            final ContainerConfiguration containerConfiguration) {
        super(log, containerConfiguration);
        this.containerService = containerService;
        this.adminService = adminService;
    }

    public void execute(Context context) {
        if (context.getEntityName() != null) {
            String slName = context.getEntityName();
            String version = context.getEntityVersion();
                // Unregister
                try {
                    this.adminService.unregisterSharedLibrary(slName + "-" + version);
                } catch (ManagementException e) {
                    this.log.error("The shared library can not be unregistered", e);
                }
                
                // Remove of the fractal installer component
                try {
                    this.containerService.removeSharedLibraryLifeCycle(slName + "-" + version);
                } catch (PetalsException e) {
                    this.log.error("The fractal shared library can not be removed",
                            e);
                }
            }
    }

    public void undo(Context context) throws Exception {
        // TODO : recreate and register the shared library
    }

}
