/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.messaging.exchange;

import java.net.URI;
import java.util.logging.Logger;

import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.ow2.petals.jbi.messaging.exchange.AbstractContextualMessageExchangeFactory;
import org.ow2.petals.jbi.messaging.exchange.ContextualMessageExchangeImplBuilder;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeWrapper;
import org.ow2.petals.microkernel.jbi.messaging.exchange.types.InOnlyImpl;
import org.ow2.petals.microkernel.jbi.messaging.exchange.types.InOptionalOutImpl;
import org.ow2.petals.microkernel.jbi.messaging.exchange.types.InOutImpl;
import org.ow2.petals.microkernel.jbi.messaging.exchange.types.RobustInOnlyImpl;

/**
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public class PersistedContextualMessageExchangeFactoryImpl extends
        AbstractContextualMessageExchangeFactory {

    private PersistedContextualMessageExchangeFactoryImpl(ServiceEndpoint consumerEndpoint,
            ContextualMessageExchangeImplBuilder messageExchangeImplBuilder, Logger logger) {
        super(consumerEndpoint, messageExchangeImplBuilder, logger);
    }

    public static final PersistedContextualMessageExchangeFactoryImpl createContextualMessageExchangeFactory(
            ServiceEndpoint consumerEndpoint, final Logger logger) {
        ContextualMessageExchangeImplBuilder messageExchangeImplBuilder = ContextualMessageExchangeImplBuilder
                .createMessageExchangeImplBuilder(consumerEndpoint);
        return new PersistedContextualMessageExchangeFactoryImpl(consumerEndpoint,
                messageExchangeImplBuilder, logger);
    }

    public static final PersistedContextualMessageExchangeFactoryImpl createContextualMessageExchangeFactoryForEndpoint(
            ServiceEndpoint consumerEndpoint, final Logger logger, ServiceEndpoint targetEndpoint) {
        ContextualMessageExchangeImplBuilder messageExchangeImplBuilder = ContextualMessageExchangeImplBuilder
                .createMessageExchangeImplBuilderForEndpoint(consumerEndpoint, targetEndpoint);
        return new PersistedContextualMessageExchangeFactoryImpl(consumerEndpoint,
                messageExchangeImplBuilder, logger);
    }

    public static final PersistedContextualMessageExchangeFactoryImpl createContextualMessageExchangeFactoryForInterface(
            ServiceEndpoint consumerEndpoint, final Logger logger, QName targetInterface) {
        ContextualMessageExchangeImplBuilder messageExchangeImplBuilder = ContextualMessageExchangeImplBuilder
                .createMessageExchangeImplBuilderForInterface(consumerEndpoint, targetInterface);
        return new PersistedContextualMessageExchangeFactoryImpl(consumerEndpoint,
                messageExchangeImplBuilder, logger);
    }

    public static final PersistedContextualMessageExchangeFactoryImpl createContextualMessageExchangeFactoryForService(
            ServiceEndpoint consumerEndpoint, final Logger logger, QName targetService) {
        ContextualMessageExchangeImplBuilder messageExchangeImplBuilder = ContextualMessageExchangeImplBuilder
                .createMessageExchangeImplBuilderForService(consumerEndpoint, targetService);
        return new PersistedContextualMessageExchangeFactoryImpl(consumerEndpoint,
                messageExchangeImplBuilder, logger);
    }

    @Override
    protected MessageExchangeWrapper createMessageExchangeDecoratorForPattern(
            MessageExchangeImpl exchange, URI pattern) {
        if (isInOnlyPattern(pattern)) {
            return new InOnlyImpl(exchange, Role.CONSUMER);
        } else if (isRobustInOnlyPattern(pattern)) {
            return new RobustInOnlyImpl(exchange, Role.CONSUMER);
        } else if (isInOutPattern(pattern)) {
            return new InOutImpl(exchange, Role.CONSUMER);
        } else if (isInOptionalOutPattern(pattern)) {
            return new InOptionalOutImpl(exchange, Role.CONSUMER);
        } else {
            return new PersistedMessageExchangeWrapper(exchange, Role.CONSUMER);
        }
    }

}
