/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.messaging.exchange;

import java.io.IOException;
import java.net.URI;
import java.sql.SQLException;
import java.util.Map;
import java.util.Set;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeWrapper;
import org.ow2.petals.microkernel.api.system.persistence.PersistenceService;

/**
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public class PersistedMessageExchangeWrapper extends MessageExchangeWrapper  { 
    
    private PersistenceService persistenceService;

    public PersistedMessageExchangeWrapper(MessageExchangeImpl me, Role observerRole) {
        super(me, observerRole);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.messaging.MessageExchange#createFault()
     */
    public Fault createFault() throws MessagingException {
        this.checkExchange();

        return super.createFault();
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.messaging.MessageExchange#createMessage()
     */
    public NormalizedMessage createMessage() throws MessagingException {
        this.checkExchange();

        return super.createMessage();
    }

    public org.ow2.petals.microkernel.api.jbi.messaging.ServiceEndpoint getConsumerEndpoint() {
        this.checkExchange();

        return (org.ow2.petals.microkernel.api.jbi.messaging.ServiceEndpoint) super
                .getConsumerEndpoint();
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.messaging.MessageExchange#getEndpoint()
     */
    public ServiceEndpoint getEndpoint() {
        this.checkExchange();

        return super.getEndpoint();
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.messaging.MessageExchange#getError()
     */
    public Exception getError() {
        this.checkExchange();

        return super.getError();
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.messaging.MessageExchange#getFault()
     */
    public Fault getFault() {
        this.checkExchange();

        return super.getFault();
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.messaging.MessageExchange#getInterfaceName()
     */
    public QName getInterfaceName() {
        this.checkExchange();

        return super.getInterfaceName();
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.messaging.MessageExchange#getMessage(java.lang.String)
     */
    public NormalizedMessage getMessage(String name) {
        this.checkExchange();

        return super.getMessage(name);
    }

    /**
     * Return the real implementation of this MessageExchange.
     * 
     * @return MessageExchangeImpl
     */
    public MessageExchangeImpl getMessageExchange() {
        this.checkExchange();

        return super.getMessageExchange();
    }

    /**
     * Get all the normalized messages
     * 
     * @return
     */
    public Map<String, NormalizedMessage> getMessages() {
        this.checkExchange();

        return super.getMessages();
    }
    /*
     * @see javax.jbi.messaging.MessageExchange#getOperation()
     */
    public QName getOperation() {
        this.checkExchange();

        return super.getOperation();
    }

    /*
     * @see javax.jbi.messaging.MessageExchange#getPattern()
     */
    public URI getPattern() {
        this.checkExchange();

        return super.getPattern();
    }

    /*
     * @see javax.jbi.messaging.MessageExchange#getProperty(java.lang.String)
     */
    public Object getProperty(String name) {
        this.checkExchange();

        return super.getProperty(name);
    }

    /*
     * @see javax.jbi.messaging.MessageExchange#getPropertyNames()
     */
    public Set<?> getPropertyNames() {
        this.checkExchange();

        return super.getPropertyNames();
    }

    /*
     * @see javax.jbi.messaging.MessageExchange#getRole()
     */
    public Role getRole() {
        this.checkExchange();

        return super.getRole();
    }

    /*
     * @see javax.jbi.messaging.MessageExchange#getService()
     */
    public QName getService() {
        this.checkExchange();

        return super.getService();
    }

    /*
     * @see javax.jbi.messaging.MessageExchange#getStatus()
     */
    public javax.jbi.messaging.ExchangeStatus getStatus() {
        this.checkExchange();

        return super.getStatus();
    }
    
    /**
     * Is terminated?
     * 
     * @return
     */
    public boolean isTerminated() {
        return super.isTerminated();
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.messaging.MessageExchange#isTransacted()
     */
    public boolean isTransacted() {
        this.checkExchange();

        return super.isTransacted();
    }

    /*
     * @see javax.jbi.messaging.MessageExchange#setEndpoint(java.lang.Exception)
     */
    public void setEndpoint(ServiceEndpoint endpoint) {
        this.checkExchange();

        super.setEndpoint(endpoint);
    }

    /*
     * @see javax.jbi.messaging.MessageExchange#setError(java.lang.Exception)
     */
    public void setError(Exception error) {
        this.checkExchange();

        super.setError(error);
    }

    /*
     * @see
     * javax.jbi.messaging.MessageExchange#setFault(javax.jbi.messaging.Fault)
     */
    public void setFault(Fault fault) throws MessagingException {
        this.checkExchange();

        super.setFault(fault);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.messaging.MessageExchange#setInterfaceName(javax.xml.namespace
     * .QName)
     */
    public void setInterfaceName(QName interfaceName) {
        this.checkExchange();

        super.setInterfaceName(interfaceName);
    }

    /*
     * @seejavax.jbi.messaging.MessageExchange#setMessage(javax.jbi.messaging.
     * NormalizedMessage, java.lang.String)
     */
    public void setMessage(NormalizedMessage msg, String name) throws MessagingException {
        this.checkExchange();

        super.setMessage(msg, name);
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.messaging.MessageExchange#setOperation(javax.xml.namespace.
     * QName)
     */
    public void setOperation(QName name) {
        this.checkExchange();

        super.setOperation(name);
    }

    /**
     * Set the persisted flag.
     * 
     * @param role
     */
    public void setPersisted(boolean persisted) {
        this.checkExchange();

      super.setPersisted(persisted);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.messaging.MessageExchange#setProperty(java.lang.String,
     * java.lang.Object)
     */
    public void setProperty(String name, Object obj) {
        this.checkExchange();

        super.setProperty(name, obj);
    }

    /**
     * Set the role.
     * 
     * @param role
     */
    public void setRole(Role role) {
        this.checkExchange();

        super.setRole(role);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.messaging.MessageExchange#setService(javax.xml.namespace.QName)
     */
    public void setService(QName service) {
        this.checkExchange();

        super.setService(service);
    }

    /*
     * (non-Javadoc)
     * 
     * @seejavax.jbi.messaging.MessageExchange#setStatus(javax.jbi.messaging.
     * ExchangeStatus)
     */
    public void setStatus(ExchangeStatus status) throws MessagingException {
        this.checkExchange();

        super.setStatus(status);
    }

    /**
     * Set the terminated flag.
     * 
     * @param terminated
     */
    public void setTerminated(boolean terminated) {
        this.checkExchange();

        super.setTerminated(terminated);
    }
    
    /**
     * Persist the wrapped exchange. <br>
     * Hold the PersistenceService reference to retrieve the exchange from the
     * persistence service when necessary.
     * 
     * @param persistenceService
     * @throws SQLException
     * @throws IOException
     */
    public void persistExchange(PersistenceService persistenceService) throws SQLException,
            IOException {
        if (this.messageExchange.isPersisted() || this.messageExchange.isMonitored()) {
            persistenceService.updateMessageExchange(this.messageExchange, true, null);
        } else {
            persistenceService.addMessageExchange(this.messageExchange, true, null);
        }
        this.messageExchange = null;
        this.persistenceService = persistenceService;
    }
    
    /**
     * Check if the exchange is in memory. if not, retrieve it from the
     * persistence manager
     * 
     * @throws SQLException
     */
    private void checkExchange() throws RuntimeException {
        if (this.messageExchange == null) {
            try {
                this.messageExchange = this.persistenceService.getMessageExchange(exchangeId);
            } catch (SQLException e) {
                throw new RuntimeException("Failed to retrieve Message Exchange with id '"
                        + exchangeId + "' from the persistence service", e);
            } catch (IOException e) {
                throw new RuntimeException("Failed to retrieve Message Exchange with id '"
                        + exchangeId + "' from the persistence service", e);
            } catch (ClassNotFoundException e) {
                throw new RuntimeException("Failed to retrieve Message Exchange with id '"
                        + exchangeId + "' from the persistence service", e);
            }
        }
    }

}
