/**
 * Copyright (c) 2008-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.security;

import java.security.Principal;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.regex.Pattern;

import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.apache.log4j.Logger;

/**
 * @author Christophe HAMERLING - EBM WebSourcing
 */
public class DefaultAuthorizationMap implements AuthorizationMap {

    private AuthorizationEntry defaultEntry;

    private final List<AuthorizationEntry> authorizationEntries;

    private static final Logger logger = Logger.getLogger(DefaultAuthorizationMap.class.getName());

    /**
     * Creates a new instance of DefaultAuthorizationMap
     */
    public DefaultAuthorizationMap(List<AuthorizationEntry> authorizationEntries) {
        this.authorizationEntries = authorizationEntries;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.security.acl.AuthorizationMap#getAcls(javax.jbi.servicedesc.ServiceEndpoint,
     *      javax.xml.namespace.QName)
     */
    public Set<Principal> getAccessControlList(ServiceEndpoint endpoint, QName operation) {
        if (logger.isDebugEnabled()) {
            logger.debug("Get Access control list for endpoint : " + endpoint.getEndpointName()
                    + ", operation : " + operation.toString());
        }

        final Set<Principal> acls = new HashSet<Principal>();
        if (this.defaultEntry != null) {
            acls.addAll(this.defaultEntry.getAcls());
        }
        for (final AuthorizationEntry entry : this.authorizationEntries) {
            if (this.match(entry, endpoint, operation)) {
                if (AuthorizationEntry.ROLE_TYPE.ADD.equals(entry.getType())) {
                    acls.addAll(entry.getAcls());
                } else if (AuthorizationEntry.ROLE_TYPE.SET.equals(entry.getType())) {
                    acls.clear();
                    acls.addAll(entry.getAcls());
                } else if (AuthorizationEntry.ROLE_TYPE.REM.equals(entry.getType())) {
                    acls.removeAll(entry.getAcls());
                }
            }
        }
        return acls;
    }

    /**
     * 
     * @param entry
     * @param endpoint
     * @param operation
     * @return
     */
    protected boolean match(AuthorizationEntry entry, ServiceEndpoint endpoint, QName operation) {
        return this.match(entry.getService(), endpoint.getServiceName())
                && this.match(entry.getEndpoint(), endpoint.getEndpointName())
                && ((entry.getOperation() == null) || (operation == null) || this.match(entry
                        .getOperation(), operation));
    }

    /**
     * 
     * @param acl
     * @param target
     * @return
     */
    private boolean match(QName acl, QName target) {
        return this.match(acl.getNamespaceURI(), target.getNamespaceURI())
                && this.match(acl.getLocalPart(), target.getLocalPart());
    }

    /**
     * 
     * @param acl
     * @param target
     * @return
     */
    private boolean match(String acl, String target) {
        return (acl == null) || acl.equals("*") || Pattern.matches(acl, target);
    }

    /**
     * @param defaultEntry
     *            the defaultEntry to set
     */
    public void setDefaultEntry(AuthorizationEntry defaultEntry) {
        this.defaultEntry = defaultEntry;
    }
}
