/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.system.classloader;

import java.net.URL;
import java.net.URLClassLoader;
import java.util.List;

import org.ow2.petals.basisapi.exception.PetalsException;

/**
 * @since Petals 1.0
 * @author Adrien LOUIS - EBM WebSourcing
 */
public interface ClassLoaderService {    
  
    /**
     * Create a classLoader which inherits of the listed Shared Libraries classLoaders.
     * @param classLoaderId
     * @param baseUrls
     * @param classpathUrls
     * @param useParentFirst
     * @param sharedLibrariesNameList
     * @return
     * @throws PetalsException
     */
    URLClassLoader createComponentClassLoader(String classLoaderId,
            URL[] baseUrls, List<String> classpathUrls, boolean useParentFirst,
            List<String> sharedLibrariesNameList) throws PetalsException;

    /**
     * Create a classLoader for a Shared Library. This classLoader will be added
     * to the Shared Libraries ClassLoaders.
     * 
     * @param baseUrls
     * @param classpathUrls
     * @param useParentFirst
     * @return
     * @throws PetalsException
     */
    URLClassLoader createSharedLibraryClassLoader(String classLoaderId,
            URL[] baseUrls, List<String> classpathUrls, boolean useParentFirst)
            throws PetalsException;
    
    /**
     * Check if the given classLoader Id exists.
     * 
     * @param classLoaderId
     * @return {@code true} if the classLoader exists
     */
    boolean containsClassLoader(String classLoaderId);

    /**
     * Remove the classLoader from the list. If the classLoader is a ClassLoader
     * for sharedLibraries, remove it from the SharedLibrariesClassLoader
     */
    void deleteClassLoader(String classLoaderId);
}
