/**
 * Copyright (c) 2009-2012 Capgemini, 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.transport.platform.nio.client;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.commons.pool.BaseKeyedPoolableObjectFactory;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.communication.topology.TopologyService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.transport.TransportException;
import org.ow2.petals.microkernel.transport.platform.nio.selector.NioSelector;

/**
 * NIO Client factory.
 * @author Christophe DENEUX - Capgemini Sud
 */
public class NioClientAgentPoolObjectFactory extends BaseKeyedPoolableObjectFactory {

    private final TopologyService topologyService;

    private final NioSelector selectorAgent;

    private final Logger logger;

    private final LoggingUtil log;

    private final long connectionTimeout;

    public NioClientAgentPoolObjectFactory(final TopologyService topologyService,
            final NioSelector selectorAgent, final long connectionTimeout, final Logger logger) {
        this.topologyService = topologyService;
        this.selectorAgent = selectorAgent;
        this.logger = logger;
        this.log = new LoggingUtil(logger);
        this.connectionTimeout = connectionTimeout;
    }

    @Override
    public Object makeObject(final Object key) throws Exception {

        final String containerName = (String) key;

        if (this.logger.isLoggable(Level.FINEST)) {
            this.log.debug("Creating new connection to container '" + containerName + "'");
        }

        final ContainerConfiguration containerConfiguration = this.topologyService
                .getContainerConfiguration(containerName);

        if (containerConfiguration == null) {
            throw new TransportException("Cannot get the information for the container "
                    + containerName);
        }

        final String host = containerConfiguration.getHost();
        final int port = containerConfiguration.getTCPPort();
        final NioClientAgent clientConnection = new NioClientAgent(this.logger, this.selectorAgent);
        if (this.logger.isLoggable(Level.FINEST)) {
            this.log.debug("Connecting to " + host + ":" + port + " (Connection timeout: "
                    + this.connectionTimeout + ")");
        }
        clientConnection.connect(host, port, this.connectionTimeout);

        if (this.logger.isLoggable(Level.FINEST)) {
            log.debug("Connection established to " + host + ":" + port);
        }
        return clientConnection;
    }

    @Override
    public void destroyObject(final Object key, final Object obj) throws Exception {
        final NioClientAgent nioClientAgent = (NioClientAgent) obj;

        if (this.logger.isLoggable(Level.FINEST)) {
            this.log.debug("The NIO client agent connected to the container '" + key
                    + "' is removed from the pool.");
        }
        nioClientAgent.disconnect();
    }

    @Override
    public boolean validateObject(Object key, Object obj) {
        final NioClientAgent nioClientAgent = (NioClientAgent) obj;

        return nioClientAgent.isUsable();
    }

}
