/**
 * Copyright (c) 2009-2012 Capgemini, 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.transport.platform.nio.server;

import java.lang.Thread.UncaughtExceptionHandler;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.atomic.AtomicInteger;

import org.ow2.petals.microkernel.api.util.LoggingUtil;

/**
 * @author Christophe DENEUX - Capgemini Sud
 */
public class DeserializerThreadFactory implements ThreadFactory {
    final ThreadGroup group;

    final AtomicInteger threadId = new AtomicInteger(0);

    private final LoggingUtil log;

    public DeserializerThreadFactory(final LoggingUtil log) {
        this.group = Thread.currentThread().getThreadGroup();
        this.log = log;
    }

    public Thread newThread(final Runnable runnable) {

        final Thread thread = new Thread(this.group, runnable, "deserializer-thread-" + this.threadId.getAndIncrement());
        thread.setUncaughtExceptionHandler(new UncaughtExceptionHandler() {

            /**
             * {@inheritDoc}
             */
            public void uncaughtException(Thread t, Throwable e) {
                DeserializerThreadFactory.this.log.error("Thread '" + t.getName() + "' threw an uncaught exception: "
                        + e.getMessage(), e);
            }
        });

        return thread;
    }

}
