/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.messaging.exchange;

import static org.junit.Assert.assertEquals;

import java.net.URI;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessageExchangeFactory;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.junit.Test;
import org.mockito.Mockito;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfOperation;

/**
 * @author Olivier Fabre - EBM WebSourcing
 */
public class PersistedMessageExchangeFactoryForEndpointImplTest extends PersistedContextualMessageExchangeFactoryImplTest {

    protected static final String PROVIDED_PROCESS_INSTANCE_ID = "a54b74d9-47cd-e8d7-12db-14562ad778cb";

    private ServiceEndpoint targetEndpoint = Mockito
            .mock(org.ow2.petals.microkernel.api.jbi.messaging.ServiceEndpoint.class);

    @Override
    protected MessageExchangeFactory createTestMessageExchangeFactory() {
        return  PersistedContextualMessageExchangeFactoryImpl.createContextualMessageExchangeFactoryForEndpoint(consumerEndpointMock, logger, targetEndpoint);
    }

    @Test
    public void testCreateExchangeFromServAndOpUseFactorySpecificTargetEp()
            throws MessagingException {
        QName serviceName = new QName("targetService");
        QName opName = new QName("targetOperation");

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(serviceName,
                opName);

        assertGeneratedExchangeUseFactorySpecificTargetEp(actualMessageExchange);
    }

    @Test
    public void testCreateExchangeFromJBIPatternUseFactorySpecificTargetEp()
            throws MessagingException {
        URI pattern = AbsItfOperation.MEPPatternConstants.IN_ONLY.value();

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(pattern);

        assertGeneratedExchangeUseFactorySpecificTargetEp(actualMessageExchange);
    }

    @Test
    public void testCreateInOnlyExchangeUseFactorySpecificTargetEp() throws MessagingException {
        MessageExchange actualMessageExchange = messageExchangeFactory.createInOnlyExchange();

        assertGeneratedExchangeUseFactorySpecificTargetEp(actualMessageExchange);
    }

    @Test
    public void testCreateInOptionalOutExchangeUseFactorySpecificTargetEp()
            throws MessagingException {
        MessageExchange actualMessageExchange = messageExchangeFactory
                .createInOptionalOutExchange();

        assertGeneratedExchangeUseFactorySpecificTargetEp(actualMessageExchange);
    }

    @Test
    public void testCreateInOutExchangeUseFactorySpecificTargetEp() throws MessagingException {
        MessageExchange actualMessageExchange = messageExchangeFactory.createInOutExchange();

        assertGeneratedExchangeUseFactorySpecificTargetEp(actualMessageExchange);
    }

    @Test
    public void testCreateRobustInOnlyExchangeUseFactorySpecificTargetEp()
            throws MessagingException {
        MessageExchange actualMessageExchange = messageExchangeFactory.createRobustInOnlyExchange();

        assertGeneratedExchangeUseFactorySpecificTargetEp(actualMessageExchange);
    }

    private void assertGeneratedExchangeUseFactorySpecificTargetEp(
            MessageExchange actualMessageExchange) {
        assertEquals("Returned exchange must target factory specific endpoint", targetEndpoint,
                actualMessageExchange.getEndpoint());
    }

}
