/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.messaging.exchange;

import static org.junit.Assert.assertEquals;

import java.net.URI;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessageExchangeFactory;
import javax.jbi.messaging.MessagingException;
import javax.xml.namespace.QName;

import org.junit.Test;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfOperation;

/**
 * @author Olivier Fabre - EBM WebSourcing
 */
public class PersistedMessageExchangeFactoryForServiceImplTest extends
        PersistedContextualMessageExchangeFactoryImplTest {

    private QName targetService = new QName("targetService");

    protected MessageExchangeFactory createTestMessageExchangeFactory() {
        return PersistedContextualMessageExchangeFactoryImpl
                .createContextualMessageExchangeFactoryForService(consumerEndpointMock, logger,
                        targetService);
    }

    @Test
    public void testCreateExchangeFromServAndOpUseFactorySpecificTargetService()
            throws MessagingException {
        QName serviceName = new QName("targetService");
        QName opName = new QName("targetOperation");

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(serviceName,
                opName);

        assertGeneratedExchangeUseFactorySpecificTargetService(actualMessageExchange);
    }

    @Test
    public void testCreateExchangeFromJBIPatternUseFactorySpecificTargetService()
            throws MessagingException {
        URI pattern = AbsItfOperation.MEPPatternConstants.IN_ONLY.value();

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(pattern);

        assertGeneratedExchangeUseFactorySpecificTargetService(actualMessageExchange);
    }

    @Test
    public void testCreateInOnlyExchangeUseFactorySpecificTargetService() throws MessagingException {
        MessageExchange actualMessageExchange = messageExchangeFactory.createInOnlyExchange();

        assertGeneratedExchangeUseFactorySpecificTargetService(actualMessageExchange);
    }

    @Test
    public void testCreateInOptionalOutExchangeUseFactorySpecificTargetService()
            throws MessagingException {
        MessageExchange actualMessageExchange = messageExchangeFactory
                .createInOptionalOutExchange();

        assertGeneratedExchangeUseFactorySpecificTargetService(actualMessageExchange);
    }

    @Test
    public void testCreateInOutExchangeUseFactorySpecificTargetService() throws MessagingException {
        MessageExchange actualMessageExchange = messageExchangeFactory.createInOutExchange();

        assertGeneratedExchangeUseFactorySpecificTargetService(actualMessageExchange);
    }

    @Test
    public void testCreateRobustInOnlyExchangeUseFactorySpecificTargetService()
            throws MessagingException {
        MessageExchange actualMessageExchange = messageExchangeFactory.createRobustInOnlyExchange();

        assertGeneratedExchangeUseFactorySpecificTargetService(actualMessageExchange);
    }

    private void assertGeneratedExchangeUseFactorySpecificTargetService(
            MessageExchange actualMessageExchange) {
        assertEquals("Returned exchange must target factory specific service", targetService,
                actualMessageExchange.getService());
    }

}
